
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Net(ApiGroup):
    def current_plmn(self) -> dict:
        return self._connection.get('net/current-plmn')

    def net_mode(self) -> dict:
        return self._connection.get('net/net-mode')

    @authorized_call
    def network(self) -> dict:
        return self._connection.get('net/network')

    def register(self) -> dict:
        return self._connection.get('net/register')

    def net_mode_list(self) -> dict:
        return self._connection.get('net/net-mode-list')

    def plmn_list(self) -> dict:
        """
    !FIXME LOL DoS! no auth required :-D
    :return:
    """
        return self._connection.get('net/plmn-list')

    def net_feature_switch(self) -> dict:
        return self._connection.get('net/net-feature-switch')


import datetime
from huawei_lte_api.ApiGroup import ApiGroup


class Host(ApiGroup):
    def info(self, datetime: datetime.datetime, platform: str, useragent: str, version: str):
        return self._connection.post('host/info', {
            'Time': datetime.strftime('%Y%m%d%H%M%S'),
            'Timezone': 'GMT{}'.format(datetime.strftime('%z')),
            'Platform': platform,
            'PlatformVer': useragent,
            'Navigator': version,
            'NavigatorVer': useragent
        })




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Cradle(ApiGroup):
    def status_info(self) ->dict:
        return self._connection.get('cradle/status-info')

    def feature_switch(self) ->dict:
        return self._connection.get('cradle/feature-switch')



from huawei_lte_api.ApiGroup import ApiGroup


class Vpn(ApiGroup):

    def feature_switch(self) -> dict:
        return self._connection.get('vpn/feature-switch')

    def br_list(self) -> dict:
        return self._connection.get('vpn/br_list')

    def ipsec_settings(self) -> dict:
        return self._connection.get('vpn/ipsec_settings')

    def l2tp_settings(self) -> dict:
        return self._connection.get('vpn/l2tp_settings')

    def pptp_settings(self) -> dict:
        return self._connection.get('vpn/pptp_settings')



import enum


@enum.unique
class ResponseEnum(enum.Enum):
    OK = 'OK'


@enum.unique
class ResponseCodeEnum(enum.IntEnum):
    ERROR_SYSTEM_UNKNOWN = 100001
    ERROR_SYSTEM_NO_SUPPORT = 100002
    ERROR_SYSTEM_NO_RIGHTS = 100003
    ERROR_SYSTEM_BUSY = 100004
    ERROR_SYSTEM_CSFR =  125002



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Security(ApiGroup):

    def bridgemode(self) -> dict:
        return self._connection.get('security/bridgemode')

    @authorized_call
    def get_firewall_switch(self) -> dict:
        return self._connection.get('security/firewall-switch')

    @authorized_call
    def set_firewall_switch(self,
                            firewall: bool=True,
                            ip_filter: bool=False,
                            wan_ping_filter: bool=True,
                            url_filter: bool=False,
                            mac_filter: bool=False
                            ) -> dict:
        return self._connection.post('security/firewall-switch', {
            'FirewallMainSwitch': int(firewall),
            'FirewallIPFilterSwitch': int(ip_filter),
            'FirewallWanPortPingSwitch': int(wan_ping_filter),
            'firewallurlfilterswitch': int(url_filter),
            'firewallmacfilterswitch': int(mac_filter)
        })

    @authorized_call
    def mac_filter(self) -> dict:
        return self._connection.get('security/mac-filter')

    @authorized_call
    def lan_ip_filter(self) -> dict:
        return self._connection.get('security/lan-ip-filter')

    @authorized_call
    def virtual_servers(self) -> dict:
        return self._connection.get('security/virtual-servers')

    @authorized_call
    def url_filter(self) -> dict:
        return self._connection.get('security/url-filter')

    @authorized_call
    def upnp(self) -> dict:
        return self._connection.get('security/upnp')



import dicttoxml
import xmltodict
import requests
import urllib.parse
import re
from huawei_lte_api.enums.client import ResponseCodeEnum
from huawei_lte_api.exceptions import \
    ResponseErrorException, \
    ResponseErrorLoginRequiredException, \
    ResponseErrorNotSupportedException, \
    ResponseErrorSystemBusyException, \
    ResponseErrorLoginCsfrException


class Connection(object):
    csfr_re = re.compile('name="csrf_token"\s+content="(\S+)"')
    cookie_jar = None
    request_verification_tokens = []

    def __init__(self, url: str):
        if not url.endswith('/'):
            raise Exception('URL must end with /')
        self.url = url
        self._initialize_csfr_tokens_and_session()

    def reload(self):
        self._initialize_csfr_tokens_and_session()

    @staticmethod
    def _create_request_xml(data: dict, dicttoxml_xargs: dict=None) -> str:
        if not dicttoxml_xargs:
            dicttoxml_xargs = {}
        return dicttoxml.dicttoxml(data, custom_root='request', **dicttoxml_xargs)

    @staticmethod
    def _process_response_xml(xml: str) -> dict:
        return xmltodict.parse(xml, dict_constructor=dict)

    @staticmethod
    def _check_response_status(data: dict):
        error_code_to_message = {
            ResponseCodeEnum.ERROR_SYSTEM_BUSY: 'System busy',
            ResponseCodeEnum.ERROR_SYSTEM_NO_RIGHTS: 'No rights (needs login)',
            ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT: 'No support',
            ResponseCodeEnum.ERROR_SYSTEM_UNKNOWN: 'Unknown',
            ResponseCodeEnum.ERROR_SYSTEM_CSFR: 'Session error'
        }

        error_code_to_exception = {
            ResponseCodeEnum.ERROR_SYSTEM_BUSY: ResponseErrorSystemBusyException,
            ResponseCodeEnum.ERROR_SYSTEM_NO_RIGHTS: ResponseErrorLoginRequiredException,
            ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT: ResponseErrorNotSupportedException,
            ResponseCodeEnum.ERROR_SYSTEM_UNKNOWN:  ResponseErrorException,
            ResponseCodeEnum.ERROR_SYSTEM_CSFR: ResponseErrorLoginCsfrException
        }
        if 'error' in data:
            error_code = int(data['error']['code'])
            if not data['error']['message']:
                message = error_code_to_message.get(error_code, 'Unknown')
            else:
                message = data['error']['message']
            raise error_code_to_exception.get(error_code, ResponseErrorException)('{}: {}'.format(error_code, message), error_code)

        return data['response'] if 'response' in data else data

    def _initialize_csfr_tokens_and_session(self):
        # Reset
        self.request_verification_tokens = []

        response = requests.get(self.url)
        self.cookie_jar = response.cookies

        csfr_tokens = self.csfr_re.findall(response.content.decode('UTF-8'))
        if len(csfr_tokens):
            self.request_verification_tokens = csfr_tokens
        else:
            self.request_verification_tokens.append(self._get_token())

    def _build_final_url(self, endpoint: str, prefix: str='api') -> str:
        return urllib.parse.urljoin(self.url + '{}/'.format(prefix), endpoint)

    def post(self,
             endpoint: str,
             data: dict=None,
             refresh_csfr: bool=False,
             prefix: str='api',
             dicttoxml_xargs: dict=None
             ) -> dict:

        headers = {
            'Content-Type': 'application/xml'
        }
        if len(self.request_verification_tokens) > 1:
            headers['__RequestVerificationToken'] = self.request_verification_tokens.pop(0)
        else:
            headers['__RequestVerificationToken'] = self.request_verification_tokens[0]

        response = requests.post(
            self._build_final_url(endpoint, prefix),
            self._create_request_xml(data, dicttoxml_xargs) if data else '',
            headers=headers,
            cookies=self.cookie_jar
        )
        response.raise_for_status()

        if response.cookies:
            self.cookie_jar = response.cookies

        data = self._check_response_status(self._process_response_xml(response.content))

        if refresh_csfr:
            self.request_verification_tokens = []

        if '__RequestVerificationTokenone' in response.headers:
            self.request_verification_tokens.append(response.headers['__RequestVerificationTokenone'])
            if '__RequestVerificationTokentwo' in response.headers:
                self.request_verification_tokens.append(response.headers['__RequestVerificationTokentwo'])
        elif '__RequestVerificationToken' in response.headers:
            self.request_verification_tokens.append(response.headers['__RequestVerificationToken'])
        else:
            raise ResponseErrorException('Failed to get CSFR from POST response headers')

        return data

    def get(self, endpoint: str, params: dict=None, prefix: str='api') -> dict:
        headers = {}
        if len(self.request_verification_tokens) == 1:
            headers['__RequestVerificationToken'] = self.request_verification_tokens[0]

        response = requests.get(
            self._build_final_url(endpoint, prefix),
            params,
            headers=headers,
            cookies=self.cookie_jar
        )

        return self._check_response_status(self._process_response_xml(response.content))

    def _get_token(self):
        data = self.get('webserver/token')
        return data['token']




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Voice(ApiGroup):
    def config(self):
        return self._connection.get('voice/config.xml', prefix='config')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call
from huawei_lte_api.enums.wlan import AuthModeEnum, WepEncryptModeEnum, WpaEncryptModeEnum


class WLan(ApiGroup):
    def wifi_feature_switch(self) -> dict:
        return self._connection.get('wlan/wifi-feature-switch')

    def station_information(self) -> dict:
        return self._connection.get('wlan/station-information')

    def basic_settings(self) -> dict:
        return self._connection.get('wlan/basic-settings')

    @authorized_call
    def set_basic_settings(self, ssid: str, hide: bool=False, wifi_restart: bool=False):
        return self._connection.post('wlan/basic-settings', {
            'WifiSsid': ssid,
            'WifiHide': hide,
            'WifiRestart': int(wifi_restart)
        })

    @authorized_call
    def security_settings(self) -> dict:
        return self._connection.get('wlan/security-settings')

    @authorized_call
    def set_security_settings(self,
                              wpa_psk: str,
                              wep_key: str='',
                              wpa_encryption_mode: WpaEncryptModeEnum=WpaEncryptModeEnum.MIX,
                              wep_encryption_mode: WepEncryptModeEnum=WepEncryptModeEnum.WEP128,
                              auth_mode: AuthModeEnum=AuthModeEnum.AUTO,
                              wifi_restart: bool=True
                              ):
        return self._connection.post('wlan/security-settings', {
            'WifiAuthmode': auth_mode.value,
            'WifiWepKey1': wep_key,
            'WifiWpaencryptionmodes': wpa_encryption_mode.value,
            'WifiBasicencryptionmodes': wep_encryption_mode.value,
            'WifiWpapsk': wpa_psk,
            'WifiRestart': int(wifi_restart)
        })

    @authorized_call
    def multi_security_settings(self) -> dict:
        return self._connection.get('wlan/multi-security-settings')

    @authorized_call
    def multi_security_settings_ex(self) -> dict:
        return self._connection.get('wlan/multi-security-settings-ex')

    @authorized_call
    def multi_basic_settings(self) -> dict:
        return self._connection.get('wlan/multi-basic-settings')

    @authorized_call
    def set_multi_basic_settings(self, clients: list) -> dict:
        """

   :param clients: list of dicts with format {'wifihostname': hostname,'WifiMacFilterMac': mac}
   :return: dict
   """
        return self._connection.post('wlan/multi-basic-settings', {
            'Ssids': {
                'Ssid': clients
            },
            'WifiRestart': 1
        })

    @authorized_call
    def host_list(self) -> dict:
        return self._connection.get('wlan/host-list')

    def handover_setting(self) -> dict:
        return self._connection.get('wlan/handover-setting')

    def set_handover_setting(self, handover: int):
        """
    G3_PREFER = 0
    WIFI_PREFER = 2
    :param handover:
    :return:
    """
        return self._connection.post('wlan/handover-setting', {
            'Handover': handover
        })

    def multi_switch_settings(self) -> dict:
        return self._connection.get('wlan/multi-switch-settings')

    @authorized_call
    def multi_macfilter_settings(self) -> dict:
        return self._connection.get('wlan/multi-macfilter-settings')

    @authorized_call
    def set_multi_macfilter_settings(self, clients: list) -> dict:
        """

        :param clients: list of dicts with format {'wifihostname': hostname,'WifiMacFilterMac': mac}
        :return: dict
        """
        return self._connection.post('wlan/multi-macfilter-settings', {
            'Ssids': {
                'Ssid': clients
            }
        })

    @authorized_call
    def mac_filter(self) -> dict:
        return self._connection.get('wlan/mac-filter')

    @authorized_call
    def set_mac_filter(self, hostname: str, mac: str):
        return self._connection.post('wlan/mac-filter', {
            'wifihostname': hostname,
            'WifiMacFilterMac': mac
        })

    @authorized_call
    def oled_showpassword(self) -> dict:
        return self._connection.get('wlan/oled-showpassword')

    @authorized_call
    def wps(self) -> dict:
        return self._connection.get('wlan/wps')

    @authorized_call
    def wps_appin(self) -> dict:
        return self._connection.get('wlan/wps-appin')

    @authorized_call
    def wps_pbc(self) -> dict:
        return self._connection.get('wlan/wps-pbc')

    @authorized_call
    def wps_switch(self) -> dict:
        return self._connection.get('wlan/wps-switch')




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Dhcp(ApiGroup):
    def settings(self) -> dict:
        return self._connection.get('dhcp/settings')

    def feature_switch(self) -> dict:
        return self._connection.get('dhcp/feature-switch')




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Global(ApiGroup):
    def module_switch(self) ->dict:
        return self._connection.get('global/module-switch')



from huawei_lte_api.ApiGroup import ApiGroup


class SdCard(ApiGroup):

    def dlna_setting(self) -> dict:
        return self._connection.get('sdcard/dlna-setting')

    def sdcard(self) -> dict:
        return self._connection.get('sdcard/sdcard')



import enum


@enum.unique
class PasswordTypeEnum(enum.IntEnum):
    BASE_64 = 0
    SHA256 = 4


@enum.unique
class LoginErrorEnum(enum.IntEnum):
    USERNAME_WRONG = 108001
    PASSWORD_WRONG = 108002
    ALREADY_LOGIN = 108003
    USERNAME_PWD_WRONG = 108006
    USERNAME_PWD_ORERRUN = 108007
    USERNAME_PWD_MODIFY = 115002


@enum.unique
class LoginStateEnum(enum.IntEnum):
    LOGGED_IN = 0
    LOGGED_OUT = -1
    REPEAT = -2


@enum.unique
class SessionErrorEnum(enum.IntEnum):
    VOICE_BUSY = 120001
    WRONG_TOKEN = 125001
    WRONG_SESSION = 125002
    WRONG_SESSION_TOKEN = 125003



import enum


@enum.unique
class AuthModeEnum(enum.Enum):
    AUTO = 'AUTO'
    OPEN = 'OPEN'
    SHARE = 'SHARE'
    WPA_PSK = 'WPA-PSK'
    WPA2_PSK = 'WPA2-PSK'
    WPA_WPA2_PSK = 'WPA/WPA2-PSK'


@enum.unique
class WpaEncryptModeEnum(enum.Enum):
    AES = 'AES'
    TKIP = 'TKIP'
    MIX = 'MIX'


@enum.unique
class WepEncryptModeEnum(enum.Enum):
    NONE = 'NONE'
    WEP = 'WEP'
    WEP64 = 'WEP64'
    WEP128 = 'WEP128'




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Pin(ApiGroup):
    def status(self) -> dict:
        return self._connection.get('pin/status')

    def simlock(self) -> dict:
        return self._connection.get('pin/simlock')